@@ -1,4 +1,4 @@
-# Copyright (c) 2008-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -20,6 +20,10 @@ from twisted.internet.interfaces import IReactorProcess, IProcessTransport
 from twisted.internet.defer import Deferred, succeed
 from twisted.internet.protocol import ProcessProtocol
 from twisted.internet.error import ProcessDone, ProcessTerminated
+from twisted.internet import _signals
+from twisted.python import runtime
+if not runtime.platform.isWindows():
+    from twisted.internet import process
 
 
 
@@ -226,6 +230,8 @@ class ProcessTestsBuilderBase(ReactorBuilder):
         if reactorClassName in skippedReactors and not hasSigInterrupt:
             raise SkipTest(
                 "%s is not supported without siginterrupt" % reactorClassName)
+        if _signals.installHandler.__name__  == "_installHandlerUsingSignal":
+            raise SkipTest("_signals._installHandlerUsingSignal doesn't support this feature")
 
         result = []
 
@@ -590,3 +596,227 @@ class PotentialZombieWarningTests(TestCase):
             "Twisted 10.0.0: There is no longer any potential for zombie "
             "process.")
         self.assertEquals(len(warnings), 1)
+
+
+
+class FakeResourceModule(object):
+    """
+    Fake version of L{resource} which hard-codes a particular rlimit for maximum
+    open files.
+    """
+    RLIMIT_NOFILE = 1
+    def getrlimit(self, no):
+        """
+        Mocks just the getrlimit method of the resource module, simulating what
+        happens on OS-X when a ridiculous number is returned.
+        """
+        return [0, 9223372036854775808]
+
+
+
+class FDDetectorTest(TestCase):
+    """
+    Tests for _FDDetector class in twisted.internet.process, which detects
+    which function to drop in place for the _listOpenFDs method.
+
+    @ivar devfs: A flag indicating whether the filesystem fake will indicate
+        that /dev/fd exists.
+
+    @ivar accurateDevFDResults: A flag indicating whether the /dev/fd fake
+        returns accurate open file information.
+
+    @ivar procfs: A flag indicating whether the filesystem fake will indicate
+        that /proc/<pid>/fd exists.
+
+    @ivar openedFile: A flag indicating whether to include an extra file
+        descriptor in the result of listing /dev/fd.
+    """
+    if runtime.platform.isWindows():
+        skip = (
+            "Process support does not need to detect open file descriptors on "
+            "Windows")
+
+    devfs = False
+    accurateDevFDResults = False
+
+    procfs = False
+    openedFile = False
+
+    savedResourceModule = None
+
+    def getpid(self):
+        """
+        Fake os.getpid, always return the same thing
+        """
+        return 123
+
+
+    def listdir(self, arg):
+        """
+        Fake os.listdir, depending on what mode we're in to simulate
+        behaviour.
+
+        @param arg: the directory to list
+        """
+        if arg == '/proc/123/fd':
+            if self.procfs:
+                return ["0","1","2"]
+            else:
+                raise OSError
+
+        if arg == '/dev/fd':
+            if self.devfs:
+                if not self.accurateDevFDResults:
+                    # Always return the same thing
+                    return ["0","1","2"]
+                else:
+                    if self.openedFile:
+                        return ["0","1","2","3"]
+                    else:
+                        return ["0","1","2"]
+            else:
+                raise OSError()
+
+
+    def openfile(self, fname, mode):
+        """
+        This is a mock for L{open}.  It just keeps track of the fact that a file
+        has opened so an extra file descriptor is found in /dev/fd.
+        """
+        self.openedFile = True
+
+
+    def saveResourceModule(self):
+        """
+        Save the resource module so we can restore it later.
+        """
+        try:
+            import resource
+        except ImportError:
+            resource = None
+        self.savedResourceModule = resource
+
+
+    def hideResourceModule(self):
+        """
+        Make the L{resource} module unimportable for the remainder of the
+        current test method.
+        """
+        sys.modules['resource'] = None
+        self.addCleanup(self.replaceResourceModule)
+
+
+    def revealResourceModule(self):
+        """
+        Make a L{FakeResourceModule} instance importable at the L{resource}
+        name.
+        """
+        sys.modules['resource'] = FakeResourceModule()
+        self.addCleanup(self.replaceResourceModule)
+
+
+    def replaceResourceModule(self):
+        """
+        Restore the original resource module to L{sys.modules}.
+        """
+        sys.modules['resource'] = self.savedResourceModule
+
+
+    def setUp(self):
+        """
+        Set up the tests, giving ourselves a detector object to play with and
+        setting up its testable knobs to refer to our mocked versions.
+        """
+        self.detector = process._FDDetector()
+        self.detector.listdir = self.listdir
+        self.detector.getpid = self.getpid
+        self.detector.openfile = self.openfile
+
+
+    def test_identityOfListOpenFDsChanges(self):
+        """
+        Check that the identity of _listOpenFDs changes after running _listOpenFDs
+        the first time, but not after the second time it's run.
+
+        In other words, check that the monkey patching actually works.
+        """
+        # Create a new instance
+        detector = process._FDDetector()
+
+        first = detector._listOpenFDs.func_name
+        detector._listOpenFDs()
+        second = detector._listOpenFDs.func_name
+        detector._listOpenFDs()
+        third = detector._listOpenFDs.func_name
+
+        self.assertNotEquals(first, second)
+        self.assertEquals(second, third)
+
+
+    def test_accurateDevFDResults(self):
+        """
+        If there is no proc filesystem and /dev/fd returns accurate open file
+        descriptor information, L{process._devFDImplementation} is chosen.  This
+        is like FreeBSD with I{fdescfs} mounted.
+        """
+        self.procfs = False
+        self.devfs = True
+        self.accurateDevFDResults = True
+        self.assertEquals(
+            self.detector._getImplementation().func_name,
+            '_devFDImplementation')
+
+
+    def test_inaccurateDevFDResults(self):
+        """
+        If there is no proc filesystem and /dev/fd returns inaccurate
+        information, L{_process._fallbackFDImplementation} is selected.  This is
+        like FreeBSD without I{fdescfs} mounted.
+        """
+        self.procfs = False
+        self.devfs = True
+        self.accurateDevFDResults = False
+        self.assertEquals(
+            self.detector._getImplementation().func_name,
+            '_fallbackFDImplementation')
+
+
+    def test_procFilesystem(self):
+        """
+        If there is a proc filesystem, L{process._procFDImplementation} is
+        selected.  This is like Linux.
+        """
+        self.devfs = False
+        self.procfs = True
+        self.assertEquals(
+            self.detector._getImplementation().func_name,
+            '_procFDImplementation')
+
+
+    def test_resourceModuleOnly(self):
+        """
+        If there is no proc filesystem and no /dev/fd but the L{resource} module
+        is available, L{process._resourceFDImplementation} is selected.  We
+        shall conjecture that this is like most proprietary UNIXes.
+        """
+        self.devfs = False
+        self.procfs = False
+        self.revealResourceModule()
+        self.assertEquals(
+            self.detector._getImplementation().func_name,
+            '_resourceFDImplementation')
+
+
+    def test_withoutResourceModule(self):
+        """
+        If there is no proc filesystem, no /dev/fd, and no L{resource} module is
+        available, L{process._fallbackFDImplementation} is selected.  This maybe
+        be like certain proprietary UNIXes without a nicely compiled Python
+        runtime.
+        """
+        self.devfs = False
+        self.procfs = False
+        self.hideResourceModule()
+        self.assertEquals(
+            self.detector._getImplementation().func_name,
+            '_fallbackFDImplementation')
